"""
termella.ansi
Definitions for ANSI escape codes.
"""

RESET = "\033[0m"

# Cursor Controls (New in v0.0.2b)
CURSOR_HIDE = "\033[?25l"
CURSOR_SHOW = "\033[?25h"

COLORS = {
    "black": "30", "red": "31", "green": "32", "yellow": "33",
    "blue": "34", "magenta": "35", "cyan": "36", "white": "37",
    "bright_red": "91", "bright_green": "92", "bright_blue": "94",
}

BG_COLORS = {
    "bg_black": "40", "bg_red": "41", "bg_green": "42",
    "bg_blue": "44", "bg_white": "47",
}

STYLES = {
    "bold": "1", "dim": "2", "italic": "3",
    "underline": "4", "blink": "5", "reverse": "7",
}


from ..core import Text

def panel(text, color="white", title=None):
    """
    Creates a simple ASCII box around text.
    """
    text = str(text)
    lines = text.split("\n")
    width = max(len(line) for line in lines) + 2
    tl, tr, bl, br, h, v= "┌", "┐", "└", "┘", "─", "│"

    t_str = f" {title} " if title else ""
    top = h * (width - len(t_str))

    print(Text(f"{tl}{t_str}{top}{tr}").style(color))
    for line in lines:
        print(Text(f"{v}{line.ljust(width)}{v}").style(color))
    print(Text(f"{bl}{h * width}{br}").style(color))


from .core import Text

def cprint(text, color=None, bg=None, styles=None, end="\n"):
    """
    Color Print: Prints text with specified formatting.

    Args:
        text (str): The text to print.
        color (str): Foreground color name (e.g., 'red', 'blue').
        bg (str): Background color name (e.g., 'bg_white').
        styles (list/str): Styles like 'bold', 'underline'.
    """
    styled_text = Text(text).style(color, bg, styles)
    print(styled_text, end=end)

def cinput(prompt_text, color="cyan", styles="bold"):
    """
    [New in v0.0.2a] Styled Input.

    Prints a styled prompt and returns the user input.
    """
    styled_prompt = Text(prompt_text).style(color=color, styles=styles)
    try:
        return input(f"{styled_prompt} \033[0m")
    except KeyboardInterrupt:
        print()
        return None


from ..core import Text

def table(data, headers=None, border_color="white"):
    """
    Renders a list of lists as a table.

    Args:
        data (list): list of rows (e.g. [['A', 'B'], ['C', 'D']])
        headers (list): List of column titles.
    """
    if not data and not headers: 
        return

    if data:
        # Calculate column widths
        cols = len(data[0])
    else:
        cols = len(headers)

    widths = [0] * cols

    all_rows = []
    if headers:
        all_rows.append(headers)
    if data:
        all_rows.extend(data)

    for row in all_rows:
        for i in range(min(len(row), cols)):
            widths[i] = max(widths[i], len(str(row[i])))

    # Formatting helpers
    def print_sep(left, mid, right, line):
        segments = [line * (w + 2) for w in widths]
        full_line = left + mid.join(segments) + right
        print(Text(full_line).style(border_color))

    # Top Border
    print_sep("┌", "┬", "┐", "─")

    # Headers
    if headers:
        row_str = "│"
        for i, cell in enumerate(headers):
            cell_content = str(cell).ljust(widths[i])
            row_str += f" {cell_content} │"
        print(Text(row_str).style(border_color, styles="bold"))
        print_sep("├", "┼", "┤", "─")

    # Data
    if not data:
        total_width = sum(widths) + (cols * 3) - 1
        print(Text("│ " + "No Data Available".center(total_width) + " │").style(border_color, styles="dim"))
    else:
        for row in data:
            row_str = "│"
            for i in range(cols):
                cell = row[i] if i < len(row) else ""
                row_str += f" {str(cell).ljust(widths[i])} │"
            print(Text(row_str).style(border_color))

    # Bottom Border
    print_sep("└", "┴", "┘", "─")


from .ansi import COLORS, BG_COLORS, STYLES, RESET

class Text:
    """Represents a styled string."""

    def __init__(self, content):
        self.content = str(content)
        self._codes = []

    def style(self, color=None, bg=None, styles=None):
        """Apply styles to the text object."""
        if color in COLORS:
            self._codes.append(COLORS[color])
        if bg in BG_COLORS:
            self._codes.append(BG_COLORS[bg])
        if styles:
            if isinstance(styles, str): styles = [styles]
            for s in styles:
                if s in STYLES: self._codes.append(STYLES[s])
        return self
    
    def __str__(self):
        """Render the final ANSI string."""
        if not self._codes: return self.content
        return f"\033[{';'.join(self._codes)}m{self.content}{RESET}"
    
    def __add__(self, other):
        return str(self) + str(other)


import sys
import time
import threading
from ..ansi import CURSOR_HIDE, CURSOR_SHOW, COLORS

class Spinner:
    """
    A context manager for a loading spinner.
    Usage:
        with Spinner("Loading..."):
            time.sleep(2)
    """
    def __init__(self, message="Loading...", delay=0.1):
        self.message = message
        self.delay = delay
        self.frames = ["⠋", "⠙", "⠹", "⠸", "⠼", "⠴", "⠦", "⠧", "⠇", "⠏"]
        self.running = False
        self.thread = None

    def _spin(self):
        idx = 0
        while self.running:
            frame = self.frames[idx % len(self.frames)]
            sys.stdout.write(f"\r{CURSOR_HIDE}\033[{COLORS['cyan']}m{frame}\033[0m {self.message}")
            sys.stdout.flush()
            idx += 1
            time.sleep(self.delay)

    def __enter__(self):
        self.running = True
        self.thread = threading.Thread(target=self._spin)
        self.thread.start()
        return self
    
    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.running = False
        if self.thread:
            self.thread.join()
        sys.stdout.write(f"\r{CURSOR_SHOW}")
        if exc_type:
            sys.stdout.write(f"\033[{COLORS['red']}m✖ {self.message} Failed!\033[0m\n")
        else:
            sys.stdout.write(f"\033[{COLORS['green']}m✔ {self.message} Done.\033[0m\n")
        sys.stdout.flush()
        return False


import sys
from ..core import Text
from ..ansi import CURSOR_HIDE, CURSOR_SHOW

def progress_bar(iteration, total, length=30, color="green", fill="█", empty="-"):
    """
    [New in v0.0.2a] Prints a progress bar to the console.
    """
    if total == 0: total = 1
    percent = ("{0:.1f}").format(100 * (iteration / float(total)))
    filled_length = int(length * iteration // total)

    filled = str(Text(fill * filled_length).style(color))
    unfilled = empty * (length - filled_length)

    # Use carriage return \r to stay on the same line
    sys.stdout.write(f'{CURSOR_HIDE}\r{filled}{unfilled} {percent}%')
    sys.stdout.flush()

    if iteration >= total:
        sys.stdout.write(f'{CURSOR_SHOW}\n')


import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="termella",
    version="0.0.3b",
    description="A Python library for rich text and beautiful formatting in the terminal.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Your Name",
    author_email="your.email@example.com",
    url="https://github.com/codewithzaqar/termella",
    license="MIT",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    python_requires=">=3.6",
    keywords="terminal, cli, colors, ansi, rich-text, formatting, ui, console",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Environment :: Console",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Terminals",
        "Topic :: Utilities",
    ],
    project_urls={
        "Bug Tracker": "https://github.com/codewithzaqar/termella/issues",
        "Source": "https://github.com/codewithzaqar/termella",
    },
)


"""
Termella - Rich text and beautiful formatting in the terminal.
Version: 0.0.3b
"""

from .printer import cprint, cinput
from .core import Text
from .widgets import panel, progress_bar, table, Spinner

__version__ = "0.0.3b"
__all__ = ["cprint", "cinput", "Text", "panel", "progress_bar", "table", "Spinner"]